#!/usr/bin/env python 

# Requires numpy and matplotlib from pylab:
from pylab import *

def plotgrid(n=20, domain='p'):
    # Plots an n by n grid of quadrilateral defined by mapping the
    # square [-1,1] x [-1,1] using the mapc2p function in this directory.
    
    import mapc2p          # this should define function mapc2p
    
    [xc,yc] = meshgrid(linspace(-1,1,n+1), linspace(-1,1,n+1))
    if domain == 'p':
	# plot in physical domain
        [xp,yp] = mapc2p.mapc2p(xc,yc)
    else:
	# plot in computational domain
        xp = xc; yp = yc
    clf()
    hold(True)
    z = 0.5*ones(xc.shape)
    z[3,:] = 0.
    z[n-4,:] = 0.
    z[:,3] = 1.
    z[:,n-4] = 1.
    cmap_dict = {}
    cmap_dict['red'] = [(0.0,1.0,1.0), (0.25,1.0,1.0), \
                        (0.75,1.0,0.0), (1.0,0.0,0.0)]
    cmap_dict['green'] = [(0.0,0.0,0.0), (0.25,0.0,1.0), \
                         (0.75,1.0,0.0), (1.0,0.0,0.0)]
    cmap_dict['blue'] = [(0.0,0.0,0.0),  (0.25,0.0,1.0), \
                         (0.75,1.0,1.0), (1.0,1.0,1.0)]
    rwb_colormap = matplotlib.colors.LinearSegmentedColormap('rwb_colormap',cmap_dict)
    pcolor(xp,yp,z, cmap=rwb_colormap, alpha=0.5)

    for i in range(n+1):
        plot(xp[i,:], yp[i,:], 'k')
        plot(xp[:,i], yp[:,i], 'k')
    gca().set_aspect('equal')
    axis([-1.1,1.1,-1.1,1.1])
    
    fname = '%sgrid.png' % domain  # = 'c' or 'p'
    savefig(fname)
    print 'Saved figure to ', fname
    return rwb_colormap
    
if __name__ == '__main__':
    # if executed from command line:
    plotgrid(20, 'c')  # in computational domain
    plotgrid(20, 'p')  # in physical domain
