def mapc2p(xc,yc):
    #
    # map [-1,1] x [-1,1] to circle of radius r1
    # mapping for Figure 3.1, radial projection, 
    # as described in Section 3.1 of 
    #    Logically Rectangular Grids and Finite Volume Methods for PDEs
    #         in Circular and Spherical Domains,
    #    by Donna A. Calhoun, Christiane Helzel, and Randall J. LeVeque,
    #    http://www.amath.washington.edu/~rjl/pubs/circles

    from numpy import abs, sqrt, where
    
    r1 = 1
    absxc = abs(xc)
    absyc = abs(yc)
    d = where(absxc>absyc, absxc, absyc)
    r = sqrt(xc**2 + yc**2)
    r = where(r>1e-10, r, 1e-10)  # to avoid divide by zero
    xp = r1 * d * xc/r
    yp = r1 * d * yc/r
    
    return xp,yp
