
def mapc2p(xc,yc):
    #
    # map [-1,1] x [-1,1] to circle of radius r1
    # mapping for Figure 3.2(a), as described in Section 3.2 of 
    #    Logically Rectangular Grids and Finite Volume Methods for PDEs
    #         in Circular and Spherical Domains,
    #    by Donna A. Calhoun, Christiane Helzel, and Randall J. LeVeque,
    #    http://www.amath.washington.edu/~rjl/pubs/circles

    from numpy import sqrt, abs, where, sign
    
    r1 = 1
    absxc = abs(xc)
    absyc = abs(yc)
    d = where(absxc>absyc, absxc, absyc)
    d = where(d>1e-10, d, 1e-10)  # to avoid divide by zero

    D = r1 * d/sqrt(2)
    R = r1 * d

    center = D - sqrt(R**2 - D**2)
    xp = D/d * abs(xc)
    yp = D/d * abs(yc)

    yp = where(abs(yc)>=abs(xc), center + sqrt(R**2 - xp**2), yp)
    xp = where(abs(xc)>=abs(yc), center + sqrt(R**2 - yp**2), xp)

    xp = sign(xc) * xp
    yp = sign(yc) * yp
    print 'done'

    return xp,yp
