def mapc2p(xc,yc):
    #
    # map [-1,1] x [-1,1] to  starlike domain.
    # mapping for Figure 8.4(a) as described in Section 8.3 of
    #    Logically Rectangular Grids and Finite Volume Methods for PDEs
    #         in Circular and Spherical Domains,
    #    by Donna A. Calhoun, Christiane Helzel, and Randall J. LeVeque,
    #    http://www.amath.washington.edu/~rjl/pubs/circles

    from numpy import abs, sqrt, where, arctan, cos
    
    r1 = 1
    absxc = abs(xc)
    absyc = abs(yc)
    d = where(absxc>absyc, absxc, absyc)
    r = sqrt(xc**2 + yc**2)
    r = where(r>1e-10, r, 1e-10)  # to avoid divide by zero
    xp = r1 * d * xc/r
    yp = r1 * d * yc/r

    denom = where(abs(xp)>1e-10, abs(xp), 1e-10)
    theta = arctan(abs(yp) / denom)
    r = 1 + .2*cos(6*theta)
    xp = xp*r
    yp = yp*r

    
    return xp,yp
